#include <math.h>
#include <stdio.h>
#include <string.h>
#include "lokglo.h"
#include "standard.h"
#include "lokstan.h"

void standard(void)
/* Standardisiert Matrix x mit Median und Dispersion */
/* Benoetigt globale Variable p, n */
/* Veraendert globale Variable x, mx, dx */
/* Benoetigt Funktionen median, disp */
/* Benoetigt NKOR, PKOR */
{
  int i,j;
  double xtw[PKOR][NKOR];

  for (i=0; i<n; ++i)
    for (j=0; j<p; ++j)
      xtw[j][i]=x[i][j];
  for (i=0; i<p; ++i)
	mx[i]=median(xtw[i],n);
  for (i=0; i<n; ++i)
  {
    for (j=0; j<p; ++j)
      xtw[j][i]=xtw[j][i]-mx[j];
  }
  for (i=0; i<p; ++i)
	dx[i]=disp(xtw[i],n);
  for (i=0; i<n; ++i)
  {
    for (j=0; j<p; ++j)
    {
      if (dx[j]>1e-8)
        xtw[j][i]=xtw[j][i]/dx[j];
    }
  }
  for (i=0; i<n; ++i)
    for (j=0; j<p; ++j)
      x[i][j]=xtw[j][i];
}

void restandard(vektor th0, vektor s[])
/* Rechnet Mittelwertvektor und Kovarianzmatrix in vorstandardisierten Zustand zurueck */
/* Benoetigt globale Variable mx, dx, p */
{
  int i,j;

  for (j=0; j<p; ++j)
  {
    if (dx[j]<=1e-8)
      dx[j]=1;
    th0[j]=th0[j]*dx[j]+mx[j];
    for (i=0; i<p; ++i)
      s[i][j]=s[i][j]*dx[i]*dx[j];
  }
}

void stanausgabe(void)
/* Ausgabe der standardisierten Werte auf Ausgabematrix */
/* Benoetigt globale Variable x, p, n, mx, dx, out, ico, varname */
{
  int i,j;

  fprintf(out,"\nStandardization to median=0, median absolute deviation =1\n");
  for (i=0; i<p; ++i)
  {
    fprintf(out,"%12s median = %16.7lf MAD = %16.7lf \n",varname[i+1],mx[i],dx[i]);
    if (dx[i]<=1e-8)
    {
      fprintf(out,"MAD too small -> no scale-standardization of this variable.\n");
      dx[i]=1;
    }
  }
  fprintf(out,"\n");
  if (ico)
    fprintf(out,"%12s ",varname[0]);
  for (j=0; j<p; ++j)
    fprintf(out,"%10s",varname[j+1]);
  fprintf(out,"\n");
  for (j=0; j<n; ++j)
  {
    if (ico)
      fprintf(out,"%12s ",id[j]);
    for (i=0; i<p; ++i)
      fprintf(out,"%10.6g ",x[j][i]);
    fprintf(out,"\n");
  }
  fprintf(out,"\n");
}

