#include <math.h>
#include <stdio.h>
#include "lokglo.h"
#include "meancov.h"
#include "matrix.h"

void meanvec (vektor xm[], int n1)
/* Berechnung des Mittelwertvektors */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable xq */
/* Benoetigt PKOR */
{
  int i, j;

  for (i=0; i<p; ++i)
  {
    xq[i]=0;
    for (j=0; j<n1; ++j)
      xq[i]+=xm[j][i];
    xq[i]/=n1;
  }
}

int schaetz (vektor xm[], int n1)
/* Lokations-, Skalenschaetzer und Mahalanobis-Abstaende */
/* Benoetigt globale Variable p, n, x */
/* Veraendert globale Variable xq, s, vr */
/* Benoetigt Funktionen meanvec, vresid, smult */
/* Benoetigt NKOR, PKOR */
{
  meanvec (xm, n1);
  vresid (xm, xq, n1);
  smult (vr, n1);
  return(mahalanobis (x, xq, n));
}

int mahalanobis(vektor x[], vektor th0, int n)
/* Berechnung der s-standardisierten Abstaende */
/* Benoetigt globale Variable p, s, chial */
/* Veraendert globale Variable r, vr */
/* Benoetigt Funktion vresid, minv, vmult, mg, jacobi */
/* Benoetigt PKOR */
{
  int i, j, k, con,invcon;
  vektor sinv[PKOR], ev[PKOR], evt[PKOR], v2, vrtsinv;

  vresid (x, th0, n);
  mg (sinv, s, p, p);
  invcon=minv (sinv, p);
  if (invcon)
  {
    for (j=0; j<n; ++j)
    {
      for (i=0; i<p; ++i)
        vrtsinv[i]=vmult(vr[j], sinv[i], p);
      r[j]=vmult(vrtsinv, vr[j], p);
    }
  }
  else   /* Falls Cov-Matrix nicht invertierbar: EW-Zerlegung, fuer Vektoren im EW>0-Raum normaler Abstand, sonst 2*chial */
  {
    mg (sinv, s, p, p);
    jacobi(sinv, ev, p);
    for (i=0; i<p; ++i)
    {
      if (fabs(sinv[i][i]>1e-7))
        sinv[i][i]=1/sinv[i][i];
      else
        sinv[i][i]=0;
    }
    for (j=0; j<n; ++j)
    {
      con=0;
      for (i=0; i<p; ++i)
      {
        if (sinv[i][i]==0)
        {
/* TZ printf ("fabs(vmult(ev[%d],vr[%d],p))= %10.7lf\n",i,j,fabs(vmult(ev[i],vr[j],p))); */
          if (fabs(vmult(ev[i],vr[j],p))>1e-7)
          {
            con=1; i=p;
          }
        } /* if sinv==0 */
      } /* for i */
      if (con)
        r[j]=2*chial;
      else
      {
        for (i=0; i<p; ++i)
          for (k=0; k<p; ++k)
            evt[i][k]=ev[k][i];
        vmmult(vr[j],evt,vrtsinv,p,p);
        vmmult(vrtsinv,sinv,v2,p,p);
        vmmult(v2,ev,vrtsinv,p,p);
        r[j]=vmult(vrtsinv,vr[j],p);
/* TZ  printf ("r[%d]= %6.4lf\n",j,r[j]); */
      } /* else (!con) */
    } /* for j */
  } /* else (!invcon) */
  return (invcon);
}

void vresid(vektor x[], vektor th0, int n)
/* Berechnung der einfachen Abstandsvektoren vom Mittelwertvektor */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable vr */
{
  int i,j;

  for (j=0; j<n; ++j)
  {
    for (i=0; i<p; ++i)
      vr[j][i]=x[j][i]-th0[i];
  }
}

void smult (vektor vr[], int n1)
/* Errechnet die Kovarianzmatrix */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable s */
{
  int i, j, k;

  for (i=0; i<p; ++i)
  {
    for (j=0; j<p; ++j)
      s[i][j]=0;
  }
  for (i=0; i<n1; ++i)
  {
    for (j=0; j<p; ++j)
    {
      for (k=0; k<p; ++k)
	s[j][k]+=(vr[i][j]*vr[i][k]);
    }
  }
  for (i=0; i<p; ++i)
  {
    for (j=0; j<p; ++j)
      s[i][j]/=(n1-1);
  }
}

